# filesequence.py
# A PyQt4 widget for selecting a sequence of numbered files according to a
# printf-style format string.

# Robert Bridson
# Released under the GPLv3

# Get the Python Qt bindings (PyQt4 or PySide)
from PyQt4.QtCore import *
from PyQt4.QtGui import *

class FileSequence(QWidget):
    """
    FileSequence displays and allows user to select a sequence of numbered files
    """

    # constants you can OR together to control buttons displayed
    openOption = 1
    saveOption = 2

    # this signal is emitted when the user changes something
    textChanged = pyqtSignal()

    # this signal is emitted when user clicks "Open"
    openClicked = pyqtSignal()

    # this signal is emitted when user clicks "Save"
    saveClicked = pyqtSignal()

    def __init__(self,
                 parent = None,
                 defaultFileNameFormat = "untitled%03d",
                 options = 0):
        QWidget.__init__(self, parent)

        # the initial default state
        self.path = QDir.currentPath()
        self.fileNameFormat = defaultFileNameFormat

        # the layout
        self.hbox = QHBoxLayout(self)

        # directory display/button
        self.pathSelect = QPushButton(self)
        self.pathSelect.setText(self.path)
        self.pathSelect.clicked.connect(self._selectDirectory)
        self.hbox.addWidget(self.pathSelect, 2)

        # file name format string
        self.formatString = QLineEdit(self)
        self.formatString.setMinimumWidth(100)
        self.formatString.setText(self.fileNameFormat)
        # validation isn't working yet... mysterious
        #self.validator = _SequenceNameValidator()
        #self.formatString.setValidator(self.validator)
        self.formatString.editingFinished.connect(self._updateFormat)
        self.hbox.addWidget(self.formatString, 1)

        # open button (if needed)
        if options & FileSequence.openOption:
            self.openButton = QPushButton(self)
            self.openButton.setText(QString("Open"))
            self.openButton.clicked.connect(self._open)
            self.hbox.addWidget(self.openButton, 0)

        # save button (if needed)
        if options & FileSequence.saveOption:
            self.saveButton = QPushButton(self)
            self.saveButton.setText(QString("Save"))
            self.saveButton.clicked.connect(self._save)
            self.hbox.addWidget(self.saveButton, 0)

    def getFileNameFormat(self):
        return str(self.path) + "/" + str(self.fileNameFormat)

    def generateFileName(self, frameNumber):
        return str(self.path) + "/" + (str(self.fileNameFormat) % frameNumber)

    def getAllExistingFileNames(self):
        return []

    def _selectDirectory(self):
        newDir = QFileDialog.getExistingDirectory(None,
                                                  QString("Select a Directory"),
                                                  self.path)
        if not newDir.isEmpty():
            self.path = newDir
            self.pathSelect.setText(self.path)
            self.pathSelect.update()
            self.textChanged.emit()

    def _updateFormat(self):
        self.fileNameFormat = self.formatString.text()
        self.textChanged.emit()

    def _open(self):
        self.openClicked.emit()

    def _save(self):
        self.saveClicked.emit()


class _SequenceNameValidator(QValidator):
    def __init__(self, parent = None):
        QValidator.__init__(self, parent)

    def validate(self, text, pos):
        try:
            instance0 = str(text) % 0
            instance1 = str(text) % 1
            if True or instance0 != instance1:
                return (QValidator.Acceptable, pos)
            else:
                return (QValidator.Invalid, pos)
        finally:
            return (QValidator.Intermediate, pos)

    def fixup(self, text):
        pass
