from PyQt4.QtCore import *
from PyQt4.QtGui import *

from imagesequence import ImageSequence
from imageview import ImageView
from framecontrol import FrameControl
from filesequence import FileSequence

class MainWindow(QMainWindow):
    """
    MainWindow is the controller/owner of the image play application.
    """

    def __init__(self):
        QMainWindow.__init__(self)

        # set up the model / document / data
        self.imgseq = ImageSequence()

        # set up the viewer
        self.fs = FileSequence(self, "untitled%03d.png",
                               FileSequence.openOption)
        self.imgview = ImageView(self)
        self.imgview.setImageSequence(self.imgseq)
        self.fc = FrameControl(self, ImageSequence.maxFrame)

        # establish GUI
        self.setWindowTitle("ImagePlay")
        self.gui = QWidget(self) # a dummy widget to hold the layout
        self.vbox = QVBoxLayout(self.gui)
        self.vbox.addWidget(self.fs, 0)
        self.vbox.addWidget(self.imgview, 1, Qt.AlignHCenter)
        self.vbox.addWidget(self.fc, 0)
        self.setCentralWidget(self.gui)

        # and connect everything up to make it work
        self._createActions()
        self._createMenus()
        self.fc.frameChanged.connect(self.imgview.setFrame)
        self.fs.openClicked.connect(self.open)

    #def closeEvent(self, event):
    #    event.accept()

    def open(self):
        self.imgseq.openImageSequence(self.fs.getFileNameFormat())
        self.fc.setLastFrame(self.imgseq.numFrames()-1)
        self.update()

    def _createActions(self):
        self.openAct = QAction("&Open", self)
        self.openAct.setShortcuts(QKeySequence.Open)
        self.openAct.triggered.connect(self.open)

        self.quitAct = QAction("&Quit ImagePlay", self)
        self.quitAct.setShortcuts(QKeySequence.Quit)
        self.quitAct.triggered.connect(self.close);

    def _createMenus(self):
        self.fileMenu = QMenu("&File", self)
        self.fileMenu.addAction(self.openAct)
        self.fileMenu.addAction(self.quitAct)
        self.menuBar().addMenu(self.fileMenu)

